#include <LiquidCrystal_I2C.h>
#include <TinyGPSPlus.h>
#include <SoftwareSerial.h>
#include <TinyGPS.h>
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_HMC5883_U.h>

#define RXPin 4
#define TXPin 3
#define GPSBaud 9600
#define ConsoleBaud 115200

/* Assign a unique ID to this sensor at the same time */
Adafruit_HMC5883_Unified mag = Adafruit_HMC5883_Unified(12345);

SoftwareSerial ss(RXPin, TXPin);  // The serial connection to the GPS device
TinyGPSPlus gps;                  // The TinyGPS++ object
int dT(5000);
float lat;
char time[9];
LiquidCrystal_I2C lcd(0x27, 20, 4);  //set the LCD address to 0x27 for a 20 chars and 4 line display


void displaySensorDetails(void) {
  sensor_t sensor;
  mag.getSensor(&sensor);
  Serial.println("------------------------------------");
  Serial.print("Sensor:       ");
  Serial.println(sensor.name);
  Serial.print("Driver Ver:   ");
  Serial.println(sensor.version);
  Serial.print("Unique ID:    ");
  Serial.println(sensor.sensor_id);
  Serial.print("Max Value:    ");
  Serial.print(sensor.max_value);
  Serial.println(" uT");
  Serial.print("Min Value:    ");
  Serial.print(sensor.min_value);
  Serial.println(" uT");
  Serial.print("Resolution:   ");
  Serial.print(sensor.resolution);
  Serial.println(" uT");
  Serial.println("------------------------------------");
  Serial.println("");
  delay(500);
}

void setup(void) {

  Serial.begin(ConsoleBaud);
  ss.begin(GPSBaud);
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("Time: ");
  lcd.setCursor(0, 1);
  lcd.print("Lat:");
  lcd.setCursor(0, 2);
  lcd.print("Long:");
  lcd.setCursor(0, 3);
  lcd.print("Alt:");

//  Serial.begin(9600);
  Serial.println("HMC5883 Magnetometer Test");
  Serial.println("");

  /* Initialise the sensor */
  if (!mag.begin()) {
    /* There was a problem detecting the HMC5883 ... check your connections */
    Serial.println("Ooops, no HMC5883 detected ... Check your wiring!");
    while (1)
      ;
  }

  /* Display some basic information on this sensor */
  displaySensorDetails();
}

void loop(void) {


  while (ss.available() > 0)
    gps.encode(ss.read());
  if (gps.location.isUpdated() || gps.altitude.isUpdated()) {
    Serial.print("Course Heading: ");
    Serial.println(gps.course.deg());  // Course in degrees (double)
    Serial.print("Number of Satellites: ");
    Serial.println(gps.satellites.value());  // Number of satellites in use (u32)
    Serial.print("Location: ");
    Serial.print(gps.location.lat(), 6);
    Serial.print(",");
    Serial.print(gps.location.lng(), 6);
    Serial.print(" Altitude:");
    Serial.println(gps.altitude.feet());
    if (gps.time.isUpdated()) {
      snprintf(time, 8, "%02d:%02d:%02d", gps.time.hour(), gps.time.minute(), gps.time.second());
      Serial.print("Time: ");
      Serial.println(time);
    }
    delay(dT);
    lcd.setCursor(5, 0);
    lcd.print(time);
    lcd.setCursor(4, 1);
    lcd.print(gps.location.lat());
    lcd.setCursor(5, 2);
    lcd.print(gps.location.lng());
    lcd.setCursor(4, 3);
    lcd.print(gps.altitude.feet());
  }



  /* Get a new sensor event */
  sensors_event_t event;
  mag.getEvent(&event);

  /* Display the results (magnetic vector values are in micro-Tesla (uT)) */
  Serial.print("X: ");
  Serial.print(event.magnetic.x);
  Serial.print("  ");
  Serial.print("Y: ");
  Serial.print(event.magnetic.y);
  Serial.print("  ");
  Serial.print("Z: ");
  Serial.print(event.magnetic.z);
  Serial.print("  ");
  Serial.println("uT");

  // Hold the module so that Z is pointing 'up' and you can measure the heading with x&y
  // Calculate heading when the magnetometer is level, then correct for signs of axis.
  float heading = atan2(event.magnetic.y, event.magnetic.x);

  // Once you have your heading, you must then add your 'Declination Angle', which is the 'Error' of the magnetic field in your location.
  // Find yours here: http://www.magnetic-declination.com/
  // Mine is: -13* 2' W, which is ~13 Degrees, or (which we need) 0.22 radians
  // If you cannot find your Declination, comment out these two lines, your compass will be slightly off.
  float declinationAngle = 0.22;
  heading += declinationAngle;

  // Correct for when signs are reversed.
  if (heading < 0)
    heading += 2 * PI;

  // Check for wrap due to addition of declination.
  if (heading > 2 * PI)
    heading -= 2 * PI;

  // Convert radians to degrees for readability.
  float headingDegrees = heading * 180 / M_PI;

  Serial.print("Heading (degrees): ");
  Serial.println(headingDegrees);

  delay(500);
}